# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = ['ResourceArgs', 'Resource']

@pulumi.input_type
class ResourceArgs:
    def __init__(__self__, *,
                 triggers: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None):
        """
        The set of arguments for constructing a Resource resource.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] triggers: A map of arbitrary strings that, when changed, will force the null resource to be replaced, re-running any associated provisioners.
        """
        if triggers is not None:
            pulumi.set(__self__, "triggers", triggers)

    @property
    @pulumi.getter
    def triggers(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        A map of arbitrary strings that, when changed, will force the null resource to be replaced, re-running any associated provisioners.
        """
        return pulumi.get(self, "triggers")

    @triggers.setter
    def triggers(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "triggers", value)


@pulumi.input_type
class _ResourceState:
    def __init__(__self__, *,
                 triggers: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None):
        """
        Input properties used for looking up and filtering Resource resources.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] triggers: A map of arbitrary strings that, when changed, will force the null resource to be replaced, re-running any associated provisioners.
        """
        if triggers is not None:
            pulumi.set(__self__, "triggers", triggers)

    @property
    @pulumi.getter
    def triggers(self) -> Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]:
        """
        A map of arbitrary strings that, when changed, will force the null resource to be replaced, re-running any associated provisioners.
        """
        return pulumi.get(self, "triggers")

    @triggers.setter
    def triggers(self, value: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]]):
        pulumi.set(self, "triggers", value)


class Resource(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 triggers: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 __props__=None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_aws as aws
        import pulumi_null as null
        import pulumi_std as std

        cluster = []
        for range in [{"value": i} for i in range(0, 3)]:
            cluster.append(aws.index.Instance(f"cluster-{range['value']}",
                ami=ami-0dcc1e21636832c5d,
                instance_type=m5.large))
        # The primary use-case for the null resource is as a do-nothing container
        # for arbitrary actions taken by a provisioner.
        #
        # In this example, three EC2 instances are created and then a
        # null_resource instance is used to gather data about all three
        # and execute a single action that affects them all. Due to the triggers
        # map, the null_resource will be replaced each time the instance ids
        # change, and thus the remote-exec provisioner will be re-run.
        cluster_resource = null.Resource("cluster", triggers={
            "cluster_instance_ids": std.join(separator=",",
                input=[__item["id"] for __item in cluster]).result,
        })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] triggers: A map of arbitrary strings that, when changed, will force the null resource to be replaced, re-running any associated provisioners.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[ResourceArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_aws as aws
        import pulumi_null as null
        import pulumi_std as std

        cluster = []
        for range in [{"value": i} for i in range(0, 3)]:
            cluster.append(aws.index.Instance(f"cluster-{range['value']}",
                ami=ami-0dcc1e21636832c5d,
                instance_type=m5.large))
        # The primary use-case for the null resource is as a do-nothing container
        # for arbitrary actions taken by a provisioner.
        #
        # In this example, three EC2 instances are created and then a
        # null_resource instance is used to gather data about all three
        # and execute a single action that affects them all. Due to the triggers
        # map, the null_resource will be replaced each time the instance ids
        # change, and thus the remote-exec provisioner will be re-run.
        cluster_resource = null.Resource("cluster", triggers={
            "cluster_instance_ids": std.join(separator=",",
                input=[__item["id"] for __item in cluster]).result,
        })
        ```

        :param str resource_name: The name of the resource.
        :param ResourceArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ResourceArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 triggers: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ResourceArgs.__new__(ResourceArgs)

            __props__.__dict__["triggers"] = triggers
        super(Resource, __self__).__init__(
            'null:index/resource:Resource',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            triggers: Optional[pulumi.Input[Mapping[str, pulumi.Input[str]]]] = None) -> 'Resource':
        """
        Get an existing Resource resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Mapping[str, pulumi.Input[str]]] triggers: A map of arbitrary strings that, when changed, will force the null resource to be replaced, re-running any associated provisioners.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ResourceState.__new__(_ResourceState)

        __props__.__dict__["triggers"] = triggers
        return Resource(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def triggers(self) -> pulumi.Output[Optional[Mapping[str, str]]]:
        """
        A map of arbitrary strings that, when changed, will force the null resource to be replaced, re-running any associated provisioners.
        """
        return pulumi.get(self, "triggers")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'GetDataSourceResult',
    'AwaitableGetDataSourceResult',
    'get_data_source',
    'get_data_source_output',
]

@pulumi.output_type
class GetDataSourceResult:
    """
    A collection of values returned by getDataSource.
    """
    def __init__(__self__, has_computed_default=None, id=None, inputs=None, outputs=None, random=None):
        if has_computed_default and not isinstance(has_computed_default, str):
            raise TypeError("Expected argument 'has_computed_default' to be a str")
        pulumi.set(__self__, "has_computed_default", has_computed_default)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if inputs and not isinstance(inputs, dict):
            raise TypeError("Expected argument 'inputs' to be a dict")
        pulumi.set(__self__, "inputs", inputs)
        if outputs and not isinstance(outputs, dict):
            raise TypeError("Expected argument 'outputs' to be a dict")
        pulumi.set(__self__, "outputs", outputs)
        if random and not isinstance(random, str):
            raise TypeError("Expected argument 'random' to be a str")
        pulumi.set(__self__, "random", random)

    @property
    @pulumi.getter(name="hasComputedDefault")
    def has_computed_default(self) -> str:
        """
        If set, its literal value will be stored and returned. If not, its value defaults to `"default"`. This argument exists primarily for testing and has little practical use.
        """
        return pulumi.get(self, "has_computed_default")

    @property
    @pulumi.getter
    @_utilities.deprecated("""This attribute is only present for some legacy compatibility issues and should not be used. It will be removed in a future version.""")
    def id(self) -> str:
        """
        This attribute is only present for some legacy compatibility issues and should not be used. It will be removed in a future version.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def inputs(self) -> Optional[Mapping[str, str]]:
        """
        A map of arbitrary strings that is copied into the `outputs` attribute, and accessible directly for interpolation.
        """
        return pulumi.get(self, "inputs")

    @property
    @pulumi.getter
    def outputs(self) -> Mapping[str, str]:
        """
        After the data source is "read", a copy of the `inputs` map.
        """
        return pulumi.get(self, "outputs")

    @property
    @pulumi.getter
    def random(self) -> str:
        return pulumi.get(self, "random")


class AwaitableGetDataSourceResult(GetDataSourceResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetDataSourceResult(
            has_computed_default=self.has_computed_default,
            id=self.id,
            inputs=self.inputs,
            outputs=self.outputs,
            random=self.random)


def get_data_source(has_computed_default: Optional[str] = None,
                    inputs: Optional[Mapping[str, str]] = None,
                    opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetDataSourceResult:
    """
    ## Example Usage


    :param str has_computed_default: If set, its literal value will be stored and returned. If not, its value defaults to `"default"`. This argument exists primarily for testing and has little practical use.
    :param Mapping[str, str] inputs: A map of arbitrary strings that is copied into the `outputs` attribute, and accessible directly for interpolation.
    """
    __args__ = dict()
    __args__['hasComputedDefault'] = has_computed_default
    __args__['inputs'] = inputs
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('null:index/getDataSource:getDataSource', __args__, opts=opts, typ=GetDataSourceResult).value

    return AwaitableGetDataSourceResult(
        has_computed_default=pulumi.get(__ret__, 'has_computed_default'),
        id=pulumi.get(__ret__, 'id'),
        inputs=pulumi.get(__ret__, 'inputs'),
        outputs=pulumi.get(__ret__, 'outputs'),
        random=pulumi.get(__ret__, 'random'))


@_utilities.lift_output_func(get_data_source)
def get_data_source_output(has_computed_default: Optional[pulumi.Input[Optional[str]]] = None,
                           inputs: Optional[pulumi.Input[Optional[Mapping[str, str]]]] = None,
                           opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetDataSourceResult]:
    """
    ## Example Usage


    :param str has_computed_default: If set, its literal value will be stored and returned. If not, its value defaults to `"default"`. This argument exists primarily for testing and has little practical use.
    :param Mapping[str, str] inputs: A map of arbitrary strings that is copied into the `outputs` attribute, and accessible directly for interpolation.
    """
    ...
